@@ -14,13 +14,14 @@ from pyxllib.prog.pupil import check_install_package
 
 check_install_package('fire')  # 自动安装依赖包
 
-from collections import Counter
+from collections import Counter, defaultdict
 from datetime import date
 import datetime
 import math
 import os
 import re
 import time
+from io import StringIO
 
 import fire
 import pandas as pd
@@ -28,11 +29,11 @@ from tqdm import tqdm
 
 from pyxllib.text.pupil import chinese2digits, grp_chinese_char
 from pyxllib.file.xlsxlib import openpyxl
-from pyxllib.file.specialist import XlPath
+from pyxllib.file.specialist import XlPath, get_encoding
+from pyxllib.prog.pupil import run_once
 from pyxllib.prog.specialist import parse_datetime, browser, TicToc, dprint
 from pyxllib.cv.rgbfmt import RgbFormatter
 from pyxllib.data.sqlite import Connection
-
 from pyxllib.ext.seleniumlib import XlChrome
 
 
@@ -206,7 +207,7 @@ class 网课考勤:
             if m := re.match(r'(\d{4}\-\d{2}\-\d{2}).+?课.*?(\d+).+?直播观看详情', f.stem):
                 stat_day, 课次 = date.fromisoformat(m.group(1)), int(m.group(2))
                 skiprows = 1
-            elif m := re.search(r'\d+届.+?(\d+).+?直播用户列表.+?(\d{4}\-\d{2}\-\d{2})', f.stem):
+            elif m := re.search(r'\d+届.+?(\d+).+?直播用户列表.+?(\d{4}\-\d{2}\-\d{2})', f.stem):  # 新版格式
                 stat_day, 课次 = date.fromisoformat(m.group(2)), int(m.group(1))
                 skiprows = 0
             else:
@@ -234,7 +235,10 @@ class 网课考勤:
         print(*set(df['用户ID']), sep='\n')
 
     def 修订(self, 学号, 课次, 完成标记, *args):
-        user_id = self.ws.cell2({'学号': 学号}, '用户ID').value
+        try:
+            user_id = self.ws.cell2({'学号': 学号}, '用户ID').value
+        except ValueError:
+            return
         if isinstance(课次, int):
             课次 = f'第{课次:02}课'
         self.异常data[课次, user_id] = 完成标记
@@ -395,9 +399,9 @@ class 网课考勤:
 
             if d:
                 msg[-1] += '\n' + ','.join(map(str, d))
-                name.append(f'第{self.当天课次}课当堂')
             else:
                 msg[-1] = title + '满勤！！！'
+            name.append(f'第{self.当天课次}课当堂')
 
         # 5 第22课的处理
         if self.觉观禅课 and self.当天课次 == 23:
@@ -503,7 +507,8 @@ class 网课考勤:
                 money = self.打卡返款[min(v - 1, 2)]
                 订单号 = ws.cell2(i, '交易订单号').value
                 cols = [订单号, money, f'{self.返款标题}返学修日志促学金', f'{订单号}_journal']
-                ls.append(','.join(map(str, cols)))
+                if cols[0]:
+                    ls.append(','.join(map(str, cols)))
             else:
                 money = 0
 
@@ -592,7 +597,7 @@ class 网课考勤:
             c2.fill = PatternFill(fgColor=color.hex[-6:], fill_type="solid")
 
         if ls:
-            ls = [x for x in ls if '无订单号' not in x]
+            ls = [x for x in ls if ('无订单号' not in x and 'None' not in x)]
 
         # 4 生成通知，及返款文件
         desc1 = '/'.join(map(str, self.打卡返款.keys()))
@@ -642,7 +647,7 @@ class 网课考勤:
         self.write_返款总计()
 
         # 3 结课
-        if self.当天课次 == (20 + len(self.视频返款)):
+        if self.当天课次 == (self.课次数 + len(self.视频返款) - 1):
             msg.append('考勤返款工作已全部完成，若对自己促学金还有疑问的近日可以再跟我反馈。')
 
         # 4 输出日报
@@ -682,6 +687,8 @@ class 网课考勤:
         pass
 
     def ensure_driver(self):
+        if not hasattr(self, 'driver'):
+            self.driver = None
         if not self.driver:
             self.driver = XlChrome()
         return self.driver
@@ -715,7 +722,8 @@ class 网课考勤:
             print(f'第{i}课', self.课程链接[i])
             driver.get('https://admin.xiaoe-tech.com/t/data_center/index')  # 必须要找个过渡页，不然不会更新课程链接
             driver.get(self.课程链接[i])
-            driver.locate_text('//*[@id="app"]/div/div/div[1]/div[2]/div[1]/div[2]', f'第{i}课')  # 检查页面
+            # 不能写'第{i}课'，会有叫'第{i}堂'等其他情况
+            driver.locate_text('//*[@id="app"]/div/div/div[1]/div[2]/div[1]/div[2]', f'{i}')  # 检查页面
 
             driver.click('//*[@id="tab-studentTab"]/span')  # 直播间用户
             driver.click('//*[@id="pane-studentTab"]/div/div[2]/div[2]/form/div[2]/button[2]/span/span')  # 导出列表
@@ -730,6 +738,7 @@ class 网课考勤:
                                        'td[9]/div/div/span').text:
                 time.sleep(1)
             else:
+                time.sleep(2)  # 不能下载太快，还是要稍微等一会
                 break
 
         # 2.2 下载表格
@@ -744,6 +753,7 @@ class 网课考勤:
 
         # 2.3 移动表格
         time.sleep(1)
+        # 这样应该也不是很泛用，有的人浏览器下载可能会放到各种自定义的其他地方
         download_path = XlPath(os.path.join(os.environ['USERPROFILE'], 'Downloads'))
         for file in files:
             src_file = (download_path / file)
@@ -1187,3 +1197,226 @@ class KqDb(Connection):
 
         # 4.2 每天打卡情况
         self.wb.save(self.outwb)
+
+
+class 课次数据:
+    """ 读取课次数据 """
+
+    def __init__(self):
+        # 将各种不同来源的数据，按用户ID分组存储起来
+        self.用户观看数据 = defaultdict(list)  # 每个用户的观看数据情况
+        self.start_day = None  # 记录所有加载文件中，最早的时间戳，大概率就是这个课次的开课时间
+
+    def add_files(self, path, pattern):
+        files = XlPath(path).select_file(pattern)
+        for f in files:
+            self.add_考勤数据(f)
+
+    def add_考勤数据(self, file):
+        """ 推荐使用这个综合接口，代替专用接口，这个接口能实现一些综合性的操作
+        保存的用户数据，存储三元数值（观看时长，进度）
+            150，表示观看时间满30分钟
+            0~100，表示有进度数据，0%~100%
+        """
+        # 0 有普通考勤表，和从圈子下载的表格两种
+        enc = get_encoding(file.read_bytes()) or 'gbk'
+        df = pd.read_csv(file, encoding=enc, encoding_errors='ignore')
+
+        if '参与状态' in df.columns:
+            self.add_圈子进度表(file)
+        else:
+            self.add_小鹅通考勤表(file)
+
+        dates = re.findall(r'(\d{4}-\d{2}-\d{2})_\d{2}-\d{2}-\d{2}', file.stem)
+        if dates:
+            file_day = datetime.datetime.strptime(dates[-1], "%Y-%m-%d").date()
+            if self.start_day is None or self.start_day > file_day:
+                self.start_day = file_day
+
+    def add_小鹅通考勤表(self, file):
+        file = XlPath(file)
+
+        df = pd.read_csv(file, encoding='utf8')
+        assert '直播观看时长(秒)' in df.columns, f'好像下载到空数据表 {file}'
+
+        for idx, x in df.iterrows():
+            data = {'文件名': XlPath(file).stem}
+            data['直播分钟'] = int(x['直播观看时长(秒)']) // 60
+            data['回放分钟'] = int(x['回放观看时长(秒)']) // 60
+            self.用户观看数据[x['用户ID']].append(data)
+
+    def add_圈子进度表(self, file):
+        """ 圈子的考勤数据比较特别，需要独立的一个体系来存储
+        大部分会有播放进度，
+        """
+        file = XlPath(file)
+        # 圈子数据目前是gbk编码，这样会有些问题。但可能哪天平台就改成utf8编码了，所以这里提前做好兼容。
+        enc = get_encoding(file.read_bytes()) or 'gbk'
+        df = pd.read_csv(file, encoding=enc, encoding_errors='ignore')
+
+        for idx, x in df.iterrows():
+            data = {'文件名': file.stem}
+
+            if '播放进度' in x:  # 一般会有详细的播放进度
+                data['百分比进度'] = int(re.search(r'\d+', x['播放进度']).group())
+            else:  # 如果没有，也可以通过参与状态知道是否完成
+                data['百分比进度'] = 100 if x['参与状态'] == '已完成' else 0
+            # 虽然"已完成"，但显示进度可能不是'100%'，要补一个修正
+            if x['参与状态'] == '已完成':
+                data['百分比进度'] = 100
+
+            self.用户观看数据[x['用户ID']].append(data)
+
+    def 禅宗考勤结果(self, user_id):
+        """ 禅宗考勤是比较简单的，不用考虑回放情况，只要在所有数据中，判断出有完成就行
+        当然，细节考虑，也还是可以详细区分不同情况。
+
+        :return:
+            str，一般是返回字符串，表示结论
+            dict，但也可以返回一个字典，表示特殊的格式设置，这种是用于单元格颜色设置
+        """
+        直播分钟, 回放分钟, 百分比进度 = 0, 0, 0
+
+        if user_id not in self.用户观看数据:
+            return {'value': f'未开始', 'color': '白色'}
+
+        # 1 遍历所有文件数据，获得所有最大值
+        data = self.用户观看数据[user_id]
+        for x in data:
+            直播分钟 = max(直播分钟, x.get('直播分钟', 0))
+            回放分钟 = max(回放分钟, x.get('回放分钟', 0))
+            百分比进度 = max(百分比进度, x.get('百分比进度', 0))
+
+        # 2 归纳出本课次考勤结论
+        yellow = RgbFormatter.from_name('鲜黄')
+        if 百分比进度 == 100 or 回放分钟 >= 30:
+            return {'value': '已完成', 'color': '鲜绿色'}
+        elif 百分比进度:
+            return {'value': f'进度{百分比进度}%', 'color': yellow.light((100 - 百分比进度) / 100)}
+        elif 回放分钟:
+            return {'value': f'观看{回放分钟}分钟', 'color': yellow.light((100 - 回放分钟) / 30)}
+        else:
+            return {'value': f'未开始', 'color': '白色'}
+
+    def 小鹅通考勤结果(self, user_id, 需求分钟=30):
+        """ 一般是把一个课次的多天数据添加到一起用，用这个接口获得该课次的观看状态
+        比如是当堂，还是第1天回放等
+
+        :param user_id: 要获得的用户信息
+        :param 需求分钟: 判断完成的依据分钟数
+        """
+        # 1 检查文件日期
+        if user_id not in self.用户观看数据:
+            return '未开始学习'
+
+        data = self.用户观看数据[user_id]
+
+        # 理论上这里文件名就应该是排好序的
+        filenames = [x['文件名'] for x in data]
+
+        for i, filename in enumerate(filenames):
+            # 后面一串是时分秒，并不提取，但是用来限定格式匹配，避免匹配错
+            dates = re.findall(r'(\d{4}-\d{2}-\d{2})_\d{2}-\d{2}-\d{2}', filename)
+            if not dates:  # 找不到时间戳的不处理
+                continue
+
+            file_date_obj = datetime.datetime.strptime(dates[0], "%Y-%m-%d").date()
+            delta_day = (file_date_obj - self.start_day).days
+
+            # 2 按顺序提取时间
+            if delta_day == 0 and data[i]['直播分钟'] >= 需求分钟:
+                return '完成当堂学习'
+            elif delta_day > 0 and data[i]['直播分钟'] + data[i]['回放分钟'] >= 需求分钟:
+                return f'第{delta_day}天回放'
+
+        if data[-1]['直播分钟'] + data[-1]['回放分钟']:
+            return f'不足{需求分钟}分钟'
+        else:
+            return '未开始学习'
+
+    def 小鹅通考勤结果2(self, user_id, 返款梯度, 要求在线分钟=30):
+        """ 输入视频返款的梯度，这个函数相比`小鹅通考勤结果`，还会返回单元格颜色格式，对应的返款额 """
+        # 1 判断课程回放是不是结束了
+        text = self.小鹅通考勤结果(user_id, 要求在线分钟)
+        if text == '未开始学习' or '不足' in text:  # 判断该课次是否已经结束了
+            delta_day = datetime.timedelta(days=len(返款梯度))
+            if (delta_day + self.start_day) <= datetime.date.today():
+                text = '未完成学习'
+
+        if text == '完成当堂学习':
+            return text, '鲜绿色', 返款梯度[0]
+        elif m := re.match(r'第(\d+)天回放', text):
+            t = int(m.group(1))
+            money = 返款梯度[t]
+            if money:
+                color = RgbFormatter.from_name('黄色')
+                color = color.light((返款梯度[0] - money) / money)  # 根据返款额度自动变浅
+            else:
+                color = '灰色'
+            return text, color, money
+        elif text == '未开始学习' or '不足' in text:
+            return text, '白色', 0
+        elif text == '未完成学习':
+            return text, '红色', 0
+        else:
+            raise ValueError
+
+
+def get_driver(_driver_store=[None]):  # trick
+    """ 考勤这边固定一个driver来使用 """
+    if _driver_store[0] is None:
+        _driver_store[0] = XlChrome()
+    if not _driver_store[0]:  # 如果驱动没了，重新启动
+        _driver_store[0] = XlChrome()
+    return _driver_store[0]
+
+
+def 登录小鹅通(name, passwd):
+    # 登录小鹅通
+    driver = get_driver()
+    driver.get('https://admin.xiaoe-tech.com/t/login#/acount')
+    driver.locate('//*[@id="common_template_mounted_el_container"]'
+                  '/div/div[1]/div[3]/div/div[4]/div/div[1]/div[1]/div/div[2]/input').send_keys(name)
+    driver.locate('//*[@id="common_template_mounted_el_container"]'
+                  '/div/div[1]/div[3]/div/div[4]/div/div[1]/div[2]/div/div/input').send_keys(passwd)
+    driver.click('//*[@id="common_template_mounted_el_container"]/div/div[1]/div[3]/div/div[4]/div/div[2]')
+
+    # 然后自己手动操作验证码
+    # 以及选择"店铺"
+
+
+def 下载课次考勤数据(课程链接, 检查文本=''):
+    # 1 遍历课程下载表格
+    driver = get_driver()
+    driver.get('https://admin.xiaoe-tech.com/t/data_center/index')  # 必须要找个过渡页，不然不会更新课程链接
+    driver.get(课程链接)
+    # 不能写'第{i}课'，会有叫'第{i}堂'等其他情况
+    if 检查文本:  # 出现指定的文本才操作下一步
+        driver.locate_text('//*[@id="app"]/div/div/div[1]/div[2]/div[1]/div[2]', 检查文本)
+    else:  # 默认等待3秒
+        time.sleep(3)
+
+    driver.click('//*[@id="tab-studentTab"]/span')  # 直播间用户
+    driver.click('//*[@id="pane-studentTab"]/div/div[2]/div[2]/form/div[2]/button[2]/span/span')  # 导出列表
+    driver.click('//*[@id="data-export-container"]/div/div[2]/div/div[2]/div[2]/button[2]/span/span')  # 导出
+
+
+def 登录微信支付():
+    driver = get_driver()
+    driver.get('https://pay.weixin.qq.com/index.php/core/home/login')
+
+
+def 聚合读取考勤数据(data_dir, name, judge_minute=30):
+    """
+    1、在目录"data_folder"下，找前缀包含name的所有文件
+    2、并将其按日期排序，整理考勤数据，判断当堂、第一天完成等
+    3、课次完成的标记，是judge_minute要满足特定分钟数
+    """
+    # 理论上获得的第一份就是当堂数据，第二份则是回放数据
+    files = list(XlPath(data_dir).rglob_files(f'*{name}*'))
+    data = 课次数据()
+    for f in files:
+        data.add_考勤数据(f)
+
+    for user_id in data.用户观看数据:
+        print(data.小鹅通考勤结果2(user_id, [100, 90, 80, 70, 0, 0]))
