@@ -441,6 +441,8 @@ class XlSSHClient(paramiko.SSHClient):
         except ScpLimitError:
             pass
 
+        return local_path
+
     def scp_put(self, local_path, remote_dir=None, *, mkdir=True, print_mode=True, limit_bytes=None, if_exists=None):
         """ 将本地的local_path上传到服务器的remote_path
 
@@ -494,6 +496,8 @@ class XlSSHClient(paramiko.SSHClient):
         except ScpLimitError:
             pass
 
+        return remote_path
+
     def scp_put_brief(self, _dir, **kwargs):
         """ 过滤一些一般不同步的文件 """
         for p in XlPath(_dir).glob('*'):
