@@ -19,9 +19,11 @@ import shutil
 import subprocess
 import tempfile
 import ujson
-from collections import defaultdict
+from collections import defaultdict, Counter
+import math
 
-import chardet
+# import chardet
+import charset_normalizer
 import qiniu
 import requests
 import yaml
@@ -30,6 +32,7 @@ from more_itertools import chunked
 import filetype
 from tqdm import tqdm
 
+from pyxllib.prog.newbie import round_int, human_readable_size
 from pyxllib.prog.pupil import is_url, is_file, DictTool
 from pyxllib.algo.pupil import Groups
 from pyxllib.file.pupil import struct_unpack, gen_file_filter
@@ -103,97 +106,28 @@ def __3_chardet():
     pass
 
 
-def get_encoding_old(bstr):
-    """ 输入二进制字符串或文本文件，返回字符编码
+def get_encoding(data,
+                 cp_isolation=('utf_8', 'gbk', 'gb18030', 'utf_16'),
+                 preemptive_behaviour=True,
+                 explain=False):
+    """ 从字节串中检测编码类型
 
-    https://www.yuque.com/xlpr/pyxllib/get_encoding
-
-    :param bstr: 二进制字符串、文本文件
-    :return: utf8, utf-8-sig, gbk, utf16
+    :param bytes data: 要检测的字节串
+    :param List[str] cp_isolation: 指定要检测的字符集编码类型列表，只检测该列表中指定的编码类型，而不是所有可能的编码类型，默认为 None
+        注意charset_normalizer是无法识别utf-8-sig的情况的。但要获得解析后的文本内容，其实也不用通过get_encoding来中转。
+    :param bool preemptive_behaviour: 指定预处理行为，如果设置为 True，将在检测之前对数据进行预处理，例如去除 BOM、转换大小写等操作，默认为 True
+    :param bool explain: 指定是否打印出检测过程的详细信息，如果设置为 True，将打印出每个 chunk 的检测结果和置信度，默认为 False
+    :return str: 检测到的编码类型，返回字符串表示
     """
-
-    # 0 检查工具
-    def inner_detect(bdata):
-        """ https://mp.weixin.qq.com/s/gSXNf3K_JWydhej8KpyhMg """
-        from chardet.universaldetector import UniversalDetector
-        detector = UniversalDetector()
-        for part in bdata.split():
-            detector.feed(part)
-            if detector.done:
-                break
-        detector.close()
-        return detector.result['encoding']
-
-    # 1 读取编码
-    if isinstance(bstr, bytes):  # 如果输入是一个二进制字符串流则直接识别
-        # encoding = chardet.detect(bstr[:maxn])['encoding']  # 截断一下，不然太长了，太影响速度
-        encoding = inner_detect(bstr)  # 截断一下，不然太长了，太影响速度
-    elif is_file(bstr):  # 如果是文件，则按二进制打开
-        # 如果输入是一个文件名则进行读取
-        if bstr.endswith('.pdf'):
-            print('二进制文件，不应该进行编码分析，暂且默认返回utf8', bstr)
-            return 'utf8'
-        with open(bstr, 'rb') as f:  # 以二进制读取文件，注意二进制没有\r\n的值
-            bstr = f.read()
-        # encoding = chardet.detect(bstr[:maxn])['encoding']
-        encoding = inner_detect(bstr)
-    else:  # 其他类型不支持
-        return 'utf8'
-    # 检测结果存储在encoding
-
-    # 2 智能适应优化，最终应该只能是gbk、utf8两种结果中的一种
-    if encoding in ('ascii', 'utf-8', 'ISO-8859-1'):
-        # 对ascii类编码，理解成是utf-8编码；ISO-8859-1跟ASCII差不多
-        encoding = 'utf8'
-    elif encoding in ('GBK', 'GB2312'):
-        encoding = 'gbk'
-    elif encoding == 'UTF-16':
-        encoding = 'utf16'
-    elif encoding == 'UTF-8-SIG':
-        # 保留原值的一些正常识别结果
-        encoding = 'utf-8-sig'
-    elif bstr.strip():  # 如果不在预期结果内，且bstr非空，则用常见的几种编码尝试
-        # print('encoding=', encoding)
-        type_ = ('utf8', 'gbk', 'utf16')
-
-        def try_encoding(bstr, encoding):
-            try:
-                bstr.decode(encoding)
-                return encoding
-            except UnicodeDecodeError:
-                return None
-
-        for t in type_:
-            encoding = try_encoding(bstr, t)
-            if encoding: break
-    else:
-        encoding = 'utf8'
-
-    return encoding
-
-
-def get_encoding(bstr, *, maxn=1000):
-    # 1 从第一个大于127的字节开始判断
-    for start_idx in range(len(bstr)):
-        if bstr[start_idx] > 127:
-            break
-    else:  # 没有>127的字节
-        return 'utf8'
-
-    # 只要截取部分子节就能大概分析出了
-    enc = chardet.detect(bstr[start_idx:start_idx + maxn])['encoding']
-
-    # 2 转换为常见编码名
-    if enc in ('utf-8', 'ascii', 'ISO-8859-1', 'Windows-1252', 'Windows-1254', 'ISO-8859-9', 'IBM866'):
-        return 'utf8'
-    elif enc in ('UTF-8-SIG',):
-        return 'utf-8-sig'
-    elif enc in ('GB2312', 'GBK'):
-        return 'gbk'
-    elif enc in ('UTF-16',):
-        return 'utf16'
-    else:
-        raise ValueError(f"{enc}: Can't get file encoding")
+    result = charset_normalizer.from_bytes(data,
+                                           cp_isolation=cp_isolation,
+                                           preemptive_behaviour=preemptive_behaviour,
+                                           explain=explain)
+    best_match = result.best()
+    if best_match:
+        return best_match.encoding
+    else:  # 注意，这个实现是有可能会找不到编码的，此时默认返回None
+        return
 
 
 def __4_file():
@@ -840,6 +774,10 @@ class File(PathBase):
         return self._path.exists()
 
 
+def make_filter():
+    """ 从filesmatch """
+
+
 class XlPath(type(pathlib.Path())):
 
     @classmethod
@@ -921,6 +859,7 @@ class XlPath(type(pathlib.Path())):
         return int(os.stat(self).st_mtime)
 
     def size(self, *, human_readable=False):
+        """ 获取文件/目录的大小 """
         if self.is_file():
             sz = os.path.getsize(self)
         elif self.is_dir():
@@ -962,6 +901,88 @@ class XlPath(type(pathlib.Path())):
         """
         return XlPath(os.path.relpath(self, str(ref_dir)))
 
+    def __contains__(self, item):
+        """ 判断item的路径是否是在self里的，不考虑item是目录、文件，还是不存在文件的路径
+        以前缀判断为准，有需要的话请自行展开resolve、expanduser再判断
+        """
+        if not self.is_file():
+            # 根据操作系统判断路径分隔符
+            separator = '\\' if os.name == 'nt' else '/'
+
+            # 获取路径对象的字符串形式
+            abs_path_str = str(self) + separator
+            item_str = str(XlPath(item))
+
+            # 判断路径字符串是否包含相对路径字符串
+            return item_str.startswith(abs_path_str) or abs_path_str == item_str
+
+    def get_line_count(self):
+        """ 统计文件的行数 """
+        with open(self, 'rb') as file:
+            line_count = 0
+            while True:
+                chunk = file.read(10 * 1024 * 1024)  # 读取 10MB 的数据块
+                if not chunk:
+                    break
+                line_count += chunk.count(b'\n')  # 计算换行符数量
+        return line_count + 1  # 添加最后一行
+
+    def split_to_dir(self, lines_per_file, dst_dir=None, encoding='utf-8'):
+        """ 将文件按行拆分到多个子文件中
+
+        :param int lines_per_file: 打算拆分的每个新文件的行数
+        :param str dst_dir: 目标目录，未输入的时候，输出到同stem明的目录下
+        :return list: 拆分的文件路径列表
+            拆分后文件名类似如下： 01.jsonl, 02.jsonl, ...
+        """
+        # 1 检查输入参数
+        if dst_dir is None:
+            # 如果未提供目标目录，则拆分的文件保存到当前工作目录
+            dst_dir = self.parent / f"{self.stem}"
+        else:
+            # 如果提供了目标目录，将拆分的文件保存到目标目录
+            dst_dir = XlPath(dst_dir)
+
+        if dst_dir.is_dir():
+            raise FileExistsError(f"目标目录已存在，若确定要重置目录，请先删除目录：{dst_dir}")
+
+        dst_dir.mkdir(parents=True, exist_ok=True)
+
+        # 2 拆分文件
+        split_files = []  # 用于保存拆分的文件路径
+        tmp_files = []  # 用于保存临时文件路径
+        outfile = None
+        filename_format = "{:04d}"
+        outfile_index = 0
+        line_counter = 0
+        suffix = self.suffix
+
+        with open(self, 'r', encoding=encoding) as f:
+            for line in f:
+                if line_counter % lines_per_file == 0:
+                    if outfile is not None:
+                        outfile.close()
+                    outfile_path = dst_dir / f"{self.stem}_{filename_format.format(outfile_index)}{suffix}"
+                    outfile = open(outfile_path, 'w', encoding='utf-8')
+                    tmp_files.append(str(outfile_path))
+                    split_files.append(outfile_path)  # 先占位，后面再填充
+                    outfile_index += 1
+                outfile.write(line)
+                line_counter += 1
+
+        if outfile is not None:
+            outfile.close()
+
+        # 3 重新设置文件名的对齐宽度
+        new_filename_format = "{:0" + str(len(str(len(split_files)))) + "d}"
+        for i, old_file in enumerate(split_files):
+            new_name = dst_dir / f"{new_filename_format.format(i)}{suffix}"
+            os.rename(old_file, new_name)
+            split_files[i] = new_name
+
+        # 返回拆分的文件路径列表
+        return split_files
+
     def __1_read_write(self):
         """ 参考标准库的
         read_bytes、read_text
@@ -970,9 +991,17 @@ class XlPath(type(pathlib.Path())):
         pass
 
     def read_text(self, encoding='utf8', errors='strict', return_mode: bool = False):
-        bstr = self.read_bytes()
-        if not encoding: encoding = get_encoding(bstr)
-        s = bstr.decode(encoding=encoding, errors=errors)
+        """
+        :param encoding: 效率拷贝，默认是设成utf8，但也可以设成None变成自动识别编码
+        """
+        if not encoding:
+            result = charset_normalizer.from_path(self, cp_isolation=('utf_8', 'gbk', 'utf_16'))
+            best_match = result.best()
+            s = str(best_match)
+            encoding = best_match.encoding
+        else:
+            with open(self, 'r', encoding=encoding) as f:
+                s = f.read()
 
         # 如果用\r\n作为换行符会有一些意外不好处理
         if '\r' in s:
@@ -995,10 +1024,8 @@ class XlPath(type(pathlib.Path())):
             注意返回的每行str，末尾都带'\n'
             但最后一行视情况可能有\n，可能没有\n
 
-        注，开发指南：不然扩展支持batch_size=-1获取所有数据，因为
-
+        注，开发指南：不然扩展支持batch_size=-1获取所有数据
         """
-
         f = open(self, 'r', encoding=encoding)
         return chunked(f, batch_size)
 
@@ -1006,6 +1033,10 @@ class XlPath(type(pathlib.Path())):
         with open(self, 'w', encoding=encoding, errors=errors, newline=newline) as f:
             return f.write(data)
 
+    def write_text_unix(self, data, encoding='utf8', errors=None, newline='\n'):
+        with open(self, 'w', encoding=encoding, errors=errors, newline=newline) as f:
+            return f.write(data)
+
     def read_pkl(self):
         with open(self, 'rb') as f:
             return pickle.load(f)
@@ -1055,6 +1086,11 @@ class XlPath(type(pathlib.Path())):
         else:
             return data
 
+    def write_jsonl(self, list_data, ensure_ascii=False):
+        """ 由于这种格式主要是跟商汤这边对接，就尽量跟它们的格式进行兼容 """
+        content = '\n'.join([json.dumps(x, ensure_ascii=ensure_ascii) for x in list_data])
+        self.write_text_unix(content + '\n')
+
     def read_csv(self, encoding='utf8', *, errors='strict', return_mode: bool = False,
                  delimiter=',', quotechar='"', **kwargs):
         """
@@ -1092,6 +1128,8 @@ class XlPath(type(pathlib.Path())):
             read_func = getattr(self, 'read_' + mode, None)
             if read_func:
                 return read_func(*args, **kwargs)
+            elif mode in ('jpg', 'png'):  # 常见的一些二进制数据
+                return self.read_bytes()
             else:
                 return self.read_text(*args, **kwargs)
         else:  # 非文件对象
@@ -1238,6 +1276,10 @@ class XlPath(type(pathlib.Path())):
         return need_run
 
     def copy(self, dst, if_exists=None):
+        """ 用于一般的文件、目录拷贝
+
+        不返回结果文件，是因为 if_exists 的逻辑比较特殊，比如skip的时候，这里不好实现返回的是最后的目标文件
+        """
         if not self.exists():
             return
 
@@ -1254,7 +1296,7 @@ class XlPath(type(pathlib.Path())):
     def rename2(self, dst, if_exists=None):
         """ 相比原版的rename，搞了更多骚操作，但性能也会略微下降，所以重写一个功能名 """
         if not self.exists():
-            return
+            return self
 
         dst = XlPath(dst)
         if self == dst:
@@ -1266,6 +1308,7 @@ class XlPath(type(pathlib.Path())):
                 tmp.rename(dst)
         elif dst.exist_preprcs(if_exists):
             self.rename(dst)
+        return dst
 
     def delete(self):
         if self.is_file():
@@ -1350,19 +1393,27 @@ class XlPath(type(pathlib.Path())):
             False, 保留第1个文件，删除其他文件
             TODO，添加其他删除模式
         :param print_mode:
-            TODO 支持html富文本显示，带超链接
+            0，不输出
+            'str'，普通文本输出（即返回的msg）
+            'html'，TODO 支持html富文本显示，带超链接
         """
         from humanfriendly import format_size
 
         def printf(*args, **kwargs):
-            if print_mode:
+            if print_mode == 'html':
+                raise NotImplementedError
+            elif print_mode:
                 print(*args, **kwargs)
+            msg.append(' '.join(args))
 
+        fmtsize = lambda x: format_size(x, binary=True)
+
+        msg = []
         files = self.glob_repeat_files(pattern, sort_mode=sort_mode, print_mode=print_mode,
                                        files=files, hash_func=hash_func)
         for i, (etag, files, _size) in enumerate(files, start=1):
             n = len(files)
-            printf(f'{i}、{etag}\t{format_size(_size)} × {n} ≈ {format_size(_size * n)}')
+            printf(f'{i}、{etag}\t{fmtsize(_size)} × {n} ≈ {fmtsize(_size * n)}')
 
             for j, f in enumerate(files, start=1):
                 if debug:
@@ -1376,15 +1427,17 @@ class XlPath(type(pathlib.Path())):
             if print_mode:
                 printf()
 
+        return msg
+
     def check_repeat_files(self, pattern='**/*', **kwargs):
         if 'debug' not in kwargs:
             kwargs['debug'] = True
-        self.delete_repeat_files(pattern, **kwargs)
+        return self.delete_repeat_files(pattern, **kwargs)
 
     def check_repeat_name_files(self, pattern='**/*', **kwargs):
         if 'hash_func' not in kwargs:
             kwargs['hash_func'] = lambda p: p.name.lower()
-        self.check_repeat_files(pattern, **kwargs)
+        return self.check_repeat_files(pattern, **kwargs)
 
     def __5_文件后缀相关功能(self):
         """ 检查目录里的各种文件情况 """
@@ -1409,18 +1462,39 @@ class XlPath(type(pathlib.Path())):
                     f1.rename2(f2, if_exists=if_exists)
                 j += 1
 
+    def _check_faker_suffix(self, file_list):
+        """ 检查文件扩展名是否匹配实际内容类型，并迭代文件列表进行处理。
+
+        :param file_list: 文件列表
+        :return: 迭代器，产生文件路径和相应的信息
+        """
+        for file_path in file_list:
+            t = filetype.guess(file_path)
+            if not t:
+                continue
+
+            ext = '.' + t.extension
+            ext0 = file_path.suffix
+
+            if ext0 in ('.docx', '.xlsx', '.pptx'):
+                ext0 = '.zip'
+            elif ext0 in ('.JPG', '.jpeg'):
+                ext0 = '.jpg'
+
+            if ext != ext0:
+                yield file_path, ext
+
     def xglob_faker_suffix_files(self, pattern='*'):
-        """ 检查文件扩展名是不是跟实际内容不匹配，有问题
+        """ 检查文件扩展名是不是跟实际内容类型不匹配，有问题
 
         注：推荐先运行 refine_files_suffix，本函数是大小写敏感，并且不会区分jpeg和jpg
         """
-        for f in self.glob_files(pattern):
-            t = filetype.guess(f)
-            if not t:
-                continue
-            ext = '.' + t.extension
-            if ext != f.suffix:
-                yield f, ext  # 原文件名，实际解析出的文件格式
+        return self._check_faker_suffix(self.glob_files(pattern))
+
+    def xglob_faker_suffix_images(self, pattern='*'):
+        """ 只检查原本就是图片名称标记的文件的相关数据正误
+        """
+        return self._check_faker_suffix(self.glob_images(pattern))
 
     def rename_faker_suffix_files(self, pattern='*', *, print_mode=True, if_exists=None, debug=False):
         for i, (f1, suffix2) in enumerate(self.xglob_faker_suffix_files(pattern), start=1):
@@ -1431,6 +1505,307 @@ class XlPath(type(pathlib.Path())):
                 f2 = f1.with_suffix(suffix2)
                 f1.rename2(f2, if_exists=if_exists)
 
+    def __6_文件夹分析诊断(self):
+        pass
+
+    def check_size(self, return_mode='str'):
+        import pandas as pd
+
+        msg = []
+        file_sizes = {}  # 缓存文件大小，避免重复计算
+        suffix_counts = Counter()
+        suffix_sizes = defaultdict(int)
+
+        dir_count, file_count = 0, 0
+        for root, dirs, files in os.walk(self):
+            dir_count += len(dirs)
+            file_count += len(files)
+            for file in files:
+                file_size = os.path.getsize(os.path.join(root, file))
+                file_sizes[(root, file)] = file_size
+
+                _, suffix = os.path.splitext(file)
+                suffix_counts[suffix] += 1
+                suffix_sizes[suffix] += file_size
+
+        sz = human_readable_size(sum(file_sizes.values()))
+        # 这里的目录指"子目录"数，不包含self
+        msg.append(f'一、目录数：{dir_count}，文件数：{file_count}，总大小：{sz}')
+
+        data = []
+        for suffix, count in suffix_counts.most_common():
+            size = suffix_sizes[suffix]
+            data.append([suffix, count, size])
+        data.sort(key=lambda x: (-x[2], -x[1], x[0]))  # 先按文件数，再按文件名排序
+        df = pd.DataFrame(data, columns=['suffix', 'count', 'size'])
+        df['size'] = [human_readable_size(x) for x in df['size']]
+        df.reset_index(inplace=True)
+        df['index'] += 1
+        msg.append('\n二、各后缀文件数')
+        msg.append(df.to_string(index=False))
+
+        if return_mode == 'str':
+            return '\n'.join(msg)
+        elif return_mode == 'list':
+            return msg
+        else:
+            return msg
+
+    def check_summary(self, print_mode=False, hash_func=None):
+        """ 对文件夹情况进行通用的状态检查
+
+        :param hash_func: 可以传入自定义的hash函数，用于第四块的重复文件运算
+            其实默认的get_etag就没啥问题，只是有时候为了性能考虑，可能会传入一个支持，提前有缓存知道etag的函数
+        """
+        if not self.is_dir():
+            return ''
+
+        def printf(s):
+            if print_mode:
+                print(s)
+            msg.append(s)
+
+        # 一 目录大小，二 各后缀文件大小
+        msg = []
+        printf('【' + self.as_posix() + '】目录检查')
+        printf('\n'.join(self.check_size('list')))
+
+        # 三 重名文件
+        printf('\n三、重名文件（忽略大小写，跨目录检查name重复情况）')
+        printf('\n'.join(self.check_repeat_name_files(print_mode=False)))
+
+        # 四 重复文件
+        printf('\n四、重复文件（etag相同）')
+        printf('\n'.join(self.check_repeat_files(print_mode=False, hash_func=hash_func)))
+
+        # 五 错误扩展名
+        printf('\n五、错误扩展名')
+        for i, (f1, suffix2) in enumerate(self.xglob_faker_suffix_files('**/*'), start=1):
+            printf(f'{i}、{f1.relpath(self)} -> {suffix2}')
+
+        # 六 文件配对
+        printf('\n六、文件配对（检查每个目录里stem名称是否配对，列出文件组成不单一的目录结构，请重点检查落单未配对的情况）')
+        prompt = False
+        for root, dirs, files in os.walk(self):
+            suffix_counts = defaultdict(list)
+            for file in files:
+                stem, suffix = os.path.splitext(file)
+                suffix_counts[stem].append(suffix)
+            suffix_counts = {k: tuple(sorted(v)) for k, v in suffix_counts.items()}
+            suffix_counts2 = {v: k for k, v in suffix_counts.items()}  # 反向存储，如果有重复v会进行覆盖
+            ct = Counter(suffix_counts.values())
+            if len(ct.keys()) > 1:
+                printf(root)
+                for k, v in ct.most_common():
+                    tag = f'\t{k}: {v}'
+                    if v == 1:
+                        tag += f'，{suffix_counts2[k]}'
+                    if len(k) > 1 and not prompt:
+                        tag += f'\t标记注解：有{v}组stem相同文件，配套有{k}这些后缀。其他标记同理。'
+                        prompt = True
+                    printf(tag)
+
+        return '\n'.join(msg)
+
+    def __7_目录复合操作(self):
+        """ 比较高级的一些目录操作功能 """
+
+    def delete_empty_subdir(self, recursive=True, topdown=False):
+        """ 删除指定目录下的所有空目录。
+
+        :param recursive: 是否递归删除所有子目录。
+        :param topdown: 是否从顶部向下遍历目录结构。
+            默认False，要先删除内部目录，再删除外部目录。
+        """
+
+        def _delete_empty_dirs(dir_path):
+            for dir_name in os.listdir(dir_path):
+                dir_fullpath = os.path.join(dir_path, dir_name)
+                if os.path.isdir(dir_fullpath):
+                    # 递归删除子目录中的空目录
+                    _delete_empty_dirs(dir_fullpath)
+                    if not os.listdir(dir_fullpath):
+                        # 删除空目录
+                        os.rmdir(dir_fullpath)
+
+        if not recursive:
+            for dirname in os.listdir(self):
+                dir_fullpath = os.path.join(self, dirname)
+                if os.path.isdir(dir_fullpath) and not os.listdir(dir_fullpath):
+                    os.rmdir(dir_fullpath)
+        else:
+            for dirpath, dirnames, filenames in os.walk(self, topdown=topdown):
+                for dirname in dirnames:
+                    dir_fullpath = os.path.join(dirpath, dirname)
+                    if not os.listdir(dir_fullpath):
+                        os.rmdir(dir_fullpath)
+                # 如果不递归删除子目录，则直接跳过
+                if not recursive:
+                    break
+
+    def flatten_directory(self, *, clear_empty_subdir=True):
+        """ 将子目录的文件全部取出来，放到外面的目录里
+
+        :param clear_empty_subdir: 移除文件后，删除空子目录
+        """
+        # 1 检查是否有重名文件，如果有重名文件则终止操作
+        if msg := self.check_repeat_name_files(print_mode=False):
+            print('\n'.join(msg))
+            raise ValueError('有重名文件，终止操作')
+
+        # 2 操作
+        self._flatten_directory_recursive(self, clear_empty_subdir)
+
+    def _flatten_directory_recursive(self, current_dir, clear_empty_subdir):
+        for name in os.listdir(current_dir):
+            path = os.path.join(current_dir, name)
+
+            if os.path.isdir(path):
+                # If it's a directory, recursively flatten it
+                self._flatten_directory_recursive(path, clear_empty_subdir)
+                if clear_empty_subdir:
+                    shutil.rmtree(path)
+            elif os.path.isfile(path):
+                # If it's a file, move it to the top-level directory
+                destination_path = os.path.join(self, name)
+                shutil.move(path, destination_path)
+
+    def _nest_directory_core(self, file_names, min_files_per_batch=None, groupby=None):
+        """ 核心方法：将文件列表按照指定规则进行分组
+
+        :param file_names: 文件列表
+        :param min_files_per_batch: 每个批次最少包含的文件数，默认为 None，即不限制最少文件数
+        :param groupby: 分组函数，用于指定按照哪个属性进行分组，默认为 None
+        :return: 分组结果列表
+        """
+        from pyxllib.algo.pupil import Groups, natural_sort
+
+        if groupby is None:
+            groupby = lambda p: p.stem.lower()
+        file_groups = Groups.groupby(file_names, groupby).data
+
+        if min_files_per_batch is None:
+            min_files_per_batch = 1
+
+        result_groups, current_group = [], []
+        for stem in natural_sort(file_groups.keys()):
+            current_group += file_groups[stem]
+            if len(current_group) >= min_files_per_batch:
+                result_groups.append(current_group)
+                current_group = []
+        if current_group:
+            result_groups.append(current_group)
+
+        return result_groups
+
+    def nest_directory(self, min_files_per_batch=None, groupby=None, batch_name=None, bias=0, tail_limit=None):
+        """ 将直接子文件按照一定规则拆分成多个batch子目录
+        注意这个功能和flatten_directory是对称的，所以函数名也是对称的
+
+        :param min_files_per_batch: 每个batch最少含有的文件数
+            None，相当于int=1的效果
+            int, 如果输入一个整数，则按照这个数量约束分成多个batch
+        :param groupby: 默认会把stem.lower()相同的强制归到一组
+            def groupby(p: XlPath) -> 分组用的key，相同key会归到同一组
+            这个不仅用于分组，返回的字符串，也会作为字典序排序的依据，如果想用自然序，记得加natural_sort_key进行转换
+        :param batch_name: 设置batch的名称，默认 'batch{}'
+        :param bias: 希望用不到这个参数，只有中途出bug，需要继续处理的时候，用来自动增加编号
+        :param tail_limit: 限制数量少于多少的batch，合并到上一个batch中
+        """
+        from pyxllib.algo.pupil import Groups
+
+        # 1 按stem分组，确定分组数
+        file_names = list(self.glob_files('*'))
+        if groupby is None:
+            groupby = lambda p: p.stem.lower()
+        file_groups = Groups.groupby(file_names, groupby).data
+
+        if min_files_per_batch is None:
+            min_files_per_batch = 1
+
+        # 2 将文件组合并为多个分组，每组至少包含 min_files_per_batch 个文件
+        result_groups, current_group = [], []
+        for stem in sorted(file_groups.keys()):  # 注意这里需要对取到的key按照自然序排序
+            current_group += file_groups[stem]
+            if len(current_group) >= min_files_per_batch:
+                result_groups.append(current_group)
+                current_group = []
+        if current_group:
+            if tail_limit is None:
+                tail_limit = min_files_per_batch // 10
+
+            if len(current_group) < tail_limit and result_groups:
+                result_groups[-1] += current_group
+            else:
+                result_groups.append(current_group)
+
+        # 3 整理实际的文件
+        if batch_name is None:
+            group_num = len(result_groups)
+            width = len(str(group_num))
+            batch_name = f'batch{{:0{width}}}'
+        for i, group in enumerate(result_groups, start=1):
+            d = self / batch_name.format(i + bias)
+            d.mkdir(exist_ok=True)
+            for f in group:
+                f.move(d / f.name)
+
+    def select_file(self, pos_filter=None, *, neg_filter=None):
+        from pyxllib.file.specialist.dirlib import Dir
+
+        d = Dir(self)
+        if pos_filter is None:
+            # 基于filesmatch的底层来实现，速度会比较慢一些，但功能丰富，不用重复造轮子
+            d = d.select('**/*', type_='file')
+        else:
+            d = d.select(pos_filter, type_='file')
+        if neg_filter is not None:
+            d = d.exclude(neg_filter)
+
+        files = [(self / f) for f in d.subs]
+        return files
+
+    def copy_file_filter(self, dst, pos_filter=None, *, neg_filter=None, if_exists=None):
+        """ 只能用于目录，在复制文件的时候，进行一定的筛选，而不是完全拷贝
+
+        :param dst: 目标目录
+        :param pos_filter: 对文件的筛选规则
+            其实常用的就'*.json'这种通配符
+            支持自定义函数，def pos_filter(p: XlPath)
+            参数详细用法参考 filesmatch，注意这里筛选器只对file启用，不会对dir启用，否则复制逻辑会非常乱
+        :param neg_filter:
+
+        注意！这个功能还是有点特殊的，不建议和XlPath.copy的接口做合并。
+
+        正向、反向两个过滤器可以组合使用，逻辑上是先用正向获取全部文件，然后扣除掉反向。
+        正向默认全选，反向默认不扣除。
+
+        注意：这样过滤后，空目录不会被拷贝。
+            如果有拷贝目录的需求，请自己另外写逻辑实现。
+            如果只是拷贝空目录结构的需求，可以使用 copy_dir_structure
+
+        >> p = XlPath('build')
+        >> p.copy_filter('build2', '*.toc')  # 复制直接子目录下的所有toc文件
+        >> p.copy_filter('build2', '**/*.toc')  # 复制所有toc文件
+        >> p.copy_filter('build2', lambda p: p.suffix == '.toc')  # 复制所有toc文件
+        """
+        files = self.select_file(pos_filter, neg_filter=neg_filter)
+
+        dst = XlPath(dst)
+        for f in files:
+            dst2 = dst / f
+            dst2.parent.mkdir(exist_ok=True)
+            (self / f).copy(dst2, if_exists=if_exists)
+
+    def copy_dir_structure(self, dst):
+        """ 只复制目录结构，不复制文件内容 """
+        dst = XlPath(dst)
+        for root, dirs, _ in os.walk(self):
+            # 构造目标路径
+            dst_dir = dst / XlPath(root).relative_to(self)
+            # 创建目录
+            dst_dir.mkdir(parents=True, exist_ok=True)
+
 
 def demo_file():
     """ File类的综合测试"""
@@ -1581,12 +1956,12 @@ def cache_file(file, make_data_func: Callable[[], Any] = None, *, reset=False, *
 
     def decorator(func):
         def wrapper(*args2, **kwargs2):
-            f = File(file)
+            f = XlPath.init(file, XlPath.tempdir())
             if f.exists() and not reset:  # 文件存在，直接读取返回
-                data = f.read(**kwargs)
+                data = f.read_auto(**kwargs)
             else:  # 文件不存在则要生成一份数据
                 data = func(*args2, **kwargs2)
-                f.write(data, **kwargs)
+                f.write_auto(data, **kwargs)
             return data
 
         return wrapper
@@ -1838,3 +2213,106 @@ def mygetfiles(root, filter_rule=None, recur=True):
     for root, _, files in myoswalk(root, filter_rule, recur):
         for f in files:
             yield root + '\\' + f
+
+
+def __6_high():
+    """ 一些高级的路径功能 """
+
+
+class DirsFileFinder:
+    """ 多目录里的文件检索类 """
+
+    def __init__(self, *dirs):
+        """ 支持按优先级输入多个目录dirs，会对这些目录里的文件进行统一检索 """
+        self.names = defaultdict(list)
+        self.stems = defaultdict(list)
+
+        for d in dirs:
+            self.add_dir(d)
+
+    def add_dir(self, p):
+        """ 添加备用检索目录
+        当前面的目录找不到匹配项的时候，会使用备用目录的文件
+        备用目录可以一直添加，有多个，优先级逐渐降低
+        """
+        files = list(XlPath(p).rglob_files())
+        for f in files:
+            self.names[f.name].append(f)
+            self.stems[f.stem].append(f)
+
+    def find_name(self, name):
+        """ 返回第一个匹配的结果 """
+        names = self.find_names(name)
+        if names:
+            return names[0]
+
+    def find_names(self, name):
+        """ 返回所有匹配的结果 """
+        return self.names[name]
+
+    def find_stem(self, stem):
+        stems = self.find_stems(stem)
+        if stems:
+            return stems[0]
+
+    def find_stems(self, stem):
+        return self.stems[stem]
+
+
+class TwinDirs:
+    def __init__(self, src_dir, dst_dir):
+        """ 一对'孪生'目录，一般是有一个src_dir，还有个同结构的dst_dir。
+        但dst_dir往往并不存在，是准备从src_dir处理过来的。
+        """
+        self.src_dir = XlPath(src_dir)
+        self.dst_dir = XlPath(dst_dir)
+        self.src_dir_finder = None
+
+    def reset_dst_dir(self):
+        """ 重置目标目录 """
+        self.dst_dir.delete()
+        self.dst_dir.mkdir()
+
+    def copy_file(self, src_file, if_exists=None):
+        """ 从src复制一个文件到dst里
+
+        :param XlPath src_file: 原文件位置（其实输入目录类型也是可以的，内部实现逻辑一致的）
+        """
+        src_file = XlPath(src_file)
+        _src_file = src_file.relpath(self.src_dir)
+        dst_file = self.dst_dir / _src_file
+        dst_dir = dst_file.parent
+        dst_dir.mkdir(exist_ok=True, parents=True)  # 确保目录结构存在
+        src_file.copy(dst_file, if_exists=if_exists)
+        return dst_file
+
+    def copy_ext_file(self, ext_file, if_exists=None):
+        """ 和copy_file区别，这里输入的ext_file是来自其他目录的文件
+        但是要在src_file里找到位置，按照结构复制到dst_dir
+        """
+        # 1 找到原始文件位置
+        ext_file = XlPath(ext_file)
+        if self.src_dir_finder is None:
+            self.src_dir_finder = DirsFileFinder(self.src_dir)
+
+        src_files = self.src_dir_finder.find_names(ext_file.name)
+        assert len(src_files) < 2, '出现多种匹配可能性，请检查目录'
+
+        if len(src_files) == 0:
+            src_files = self.src_dir_finder.find_stems(ext_file.stem)
+            assert len(src_files), '没有找到可匹配的文件'
+
+        src_file = src_files[0]
+
+        # 2 复制文件
+        _src_file = src_file.relpath(self.src_dir)
+        dst_file = self.dst_dir / _src_file
+        dst_dir = dst_file.parent
+        dst_dir.mkdir(exist_ok=True, parents=True)  # 确保目录结构存在
+        ext_file.copy(dst_file, if_exists=if_exists)
+        print(ext_file, '->', dst_file)
+        return dst_file
+
+    def copy_dir_structure(self):
+        """ 复制目录结构 """
+        self.src_dir.copy_dir_structure(self.dst_dir)
