@@ -11,6 +11,7 @@ from collections import Counter
 import ctypes
 import datetime
 import functools
+import hashlib
 import inspect
 import io
 import itertools
@@ -83,14 +84,6 @@ def is_file(arg, exists=True):
     return os.path.isfile(arg)
 
 
-def get_hostname():
-    return socket.getfqdn()
-
-
-def get_username():
-    return os.path.split(os.path.expanduser('~'))[-1]
-
-
 def len_in_dim2_min(arr):
     """ 计算类List结构在第2维上的最小长度
 
@@ -411,7 +404,7 @@ def check_install_package(package, speccal_install_name=None, *, user=False):
 
 
 def run_once(distinct_mode=0, *, limit=1):
-    """
+    """ 装饰器，装饰的函数在一次程序里其实只会运行一次
     :param int|str distinct_mode:
         0，默认False，不区分输入的参数值（包括cls、self），强制装饰的函数只运行一次
         'str'，设为True或1时，仅以字符串化的差异判断是否是重复调用，参数不同，会判断为不同的调用，每种调用限制最多执行limit次
@@ -422,7 +415,7 @@ def run_once(distinct_mode=0, *, limit=1):
             使用该模式，首参数会ignore忽略，只比较第2个开始之后的参数
         func等callable类型的对象也行，是使用run_once装饰器的简化写法
     :param limit: 默认只会执行一次，该参数可以提高限定的执行次数，一般用不到，用于兼容旧的 limit_call_number 装饰器
-    Returns: 返回decorator
+    returns: 返回decorator
     """
     if callable(distinct_mode):
         # @run_once，没写括号的时候去装饰一个函数，distinct_mode传入的是一个函数func
@@ -862,3 +855,131 @@ def hide_console_window():
     SW_HIDE = 0
     hWnd = kernel32.GetConsoleWindow()
     user32.ShowWindow(hWnd, SW_HIDE)
+
+
+def get_installed_packages():
+    """ 使用pip list获取当前环境安装了哪些包 """
+    output = subprocess.check_output(["pip", "list"], universal_newlines=True)
+    packages = [line.split()[0] for line in output.split("\n")[2:] if line]
+    return packages
+
+
+class OutputLogger(logging.Logger):
+    """
+    我在jupyter写代码，经常要print输出一些中间结果。
+    但是当结果很多的时候，又要转存到文件里保存起来查看。
+    要保存到文件时，和普通的print写法是不一样的，一般要新建立一个ls = []的变量。
+    然后print改成ls.append操作，会很麻烦。
+
+    就想着能不能自己定义一个类，支持.print方法，不仅能实现正常的输出控制台的功能。
+    也能在需要的时候指定文件路径，会自动将结果存储到文件中。
+    """
+
+    def __init__(self, name='OutputLogger', log_file=None, output_to_console=True):
+        """
+        :param str name: 记录器的名称。默认为 'OutputLogger'。
+        :param log_file: 日志文件的路径。默认为 None，表示不输出到文件。
+        :param bool output_to_console: 是否输出到命令行。默认为 True。
+        """
+        super().__init__(name)
+
+        self.output_to_console = output_to_console
+        self.log_file = log_file
+
+        self.setLevel(logging.INFO)
+        # 创建格式化器
+        formatter = logging.Formatter('%(asctime)s - %(levelname)s - %(filename)s/%(lineno)d - %(message)s',
+                                      '%Y-%m-%d %H:%M:%S')
+
+        # 提前重置为空文件
+        with open(log_file, 'w') as f:
+            f.write('')
+
+        # 创建文件日志处理器
+        if log_file:
+            file_handler = logging.FileHandler(log_file, encoding='utf-8')
+            file_handler.setLevel(logging.INFO)
+            file_handler.setFormatter(formatter)
+            self.addHandler(file_handler)
+
+        # 创建命令行日志处理器
+        if output_to_console:
+            console_handler = logging.StreamHandler()
+            console_handler.setLevel(logging.INFO)
+            console_handler.setFormatter(formatter)
+            self.addHandler(console_handler)
+
+    def print(self, *args, **kwargs):
+        msg = print2string(*args, **kwargs)
+
+        if self.output_to_console:
+            print(msg, end='')
+
+        if self.log_file:
+            with open(self.log_file, 'a', encoding='utf-8') as f:
+                f.write(msg)
+
+        return msg
+
+
+class MultiProcessLauncher:
+    """ 注意这个类暂时只有linux能使用 """
+
+    def __init__(self):
+        self.workers = []
+
+    def add_process(self, cmd, name=None, **kwargs):
+        if name is None:
+            name = cmd.split()[0]
+
+        p = subprocess.Popen(cmd.split(), preexec_fn=self._set_pdeathsig(signal.SIGTERM))
+        worker = {'name': name, 'process': p, 'command': cmd}
+        worker.update(kwargs)
+        self.workers.append(worker)
+
+    def add_python_module(self, module, args='', name=None):
+        """ 添加并启动一个Python模块作为后台进程。
+
+        :param module: 要执行的Python模块名（python -m 后面的部分）
+        :param args: 模块的参数
+        :param name: 进程的名称，默认为模块名
+        """
+        cmd = f'{sys.executable} -m {module} {args}'
+        self.add_process(cmd, name=name)
+
+    def stop_all(self):
+        """停止所有后台进程"""
+        for worker in self.workers:
+            worker.process.terminate()
+        self.workers = []
+
+    def _set_pdeathsig(self, sig=signal.SIGTERM):
+        """ 在主服务退出时，这些进程也会全部自动关闭 """
+
+        def callable():
+            libc = ctypes.CDLL("libc.so.6")
+            return libc.prctl(1, sig)
+
+        return callable
+
+
+def xlmd5(content):
+    if isinstance(content, str):
+        content = content.encode('utf-8')
+    elif not isinstance(content, bytes):
+        content = str(content).encode('utf-8')
+
+    if len(content) <= 32:  # 32位以下的字符串，直接返回
+        return content
+    else:
+        return hashlib.md5(content).hexdigest()
+
+
+@run_once()
+def get_hostname():
+    return socket.getfqdn()
+
+
+@run_once()
+def get_username():
+    return os.path.split(os.path.expanduser('~'))[-1]
