@@ -12,6 +12,7 @@ import os
 import subprocess
 import sys
 import datetime
+import platform
 
 import pandas as pd
 from bs4 import BeautifulSoup
@@ -21,7 +22,7 @@ from pyxllib.prog.pupil import dprint, func_input_message, is_url, is_file
 from pyxllib.prog.specialist.common import TypeConvert, NestedDict, KeyValuesCounter, dataframe_str
 from pyxllib.prog.specialist.tictoc import TicToc
 from pyxllib.text.pupil import ensure_gbk, shorten
-from pyxllib.file.specialist.dirlib import File, Dir, get_etag
+from pyxllib.file.specialist.dirlib import File, Dir, get_etag, XlPath
 
 
 def getasizeof(*objs, **opts):
@@ -145,17 +146,24 @@ class Browser(Explorer):
         :param shell:
         """
         if app is None:
-            # 智能判断环境变量，选择存在的浏览器，我的偏好 chrome > msedge
-            paths = os.environ['PATH']
-            # 220107周五10:34，不能写完整路径，有x86和x64版本，路径会有区别
-            chrome_dir = r'Google\Chrome\Application'
-            msedge_dir = r'Microsoft\Edge\Application'
-            if chrome_dir in paths:
-                app = 'chrome'
-            elif msedge_dir in paths:
-                app = 'msedge'
-            else:  # 默认使用谷歌。之前试过explorer不行~~
-                app = 'C:/Program Files/Google/Chrome/Application/chrome.exe'
+            if platform.system() == 'Windows':
+                paths = os.environ['PATH']
+                chrome_dir = r'Google\Chrome\Application'
+                msedge_dir = r'Microsoft\Edge\Application'
+                if chrome_dir in paths:
+                    app = 'chrome'
+                elif msedge_dir in paths:
+                    app = 'msedge'
+                else:  # 默认使用谷歌。之前试过explorer不行~~
+                    app = 'C:/Program Files/Google/Chrome/Application/chrome.exe'
+            elif platform.system() == 'Linux':  # Linux系统（包括Ubuntu）
+                # 可以在这里添加对应的Unix-like系统浏览器的命令行名称
+                # 这里默认设置为 'google-chrome'，如果你想使用其他的浏览器，例如Firefox，可以修改为 'firefox'
+                app = 'google-chrome'
+            else:
+                # 其他系统的处理
+                pass
+        super().__init__(app, shell)
         super().__init__(app, shell)
 
     @classmethod
@@ -199,6 +207,8 @@ class Browser(Explorer):
                 file = File(name, Dir.TEMP, suffix='.html').to_str()
             arg.render(path=str(file))
         else:  # 不在预设格式里的数据，转成普通的txt查看
+            # if File.safe_init(arg).exists():
+            #     file = arg
             if file is None:
                 file = File(..., Dir.TEMP, suffix='.txt').write(arg)
                 file = file.rename(get_etag(str(file)) + file.suffix, if_exists='replace')
@@ -223,7 +233,10 @@ class Browser(Explorer):
         :param file: 默认可以不输入，会按七牛的etag哈希值生成临时文件
             如果输入，则按照指定的名称生成文件
         """
-        file = str(self.to_brower_file(arg, file, clsmsg=clsmsg, to_html_args=to_html_args))
+        if XlPath.safe_init(arg).is_file():
+            file = arg
+        else:
+            file = str(self.to_brower_file(arg, file, clsmsg=clsmsg, to_html_args=to_html_args))
         super().__call__(str(file), wait=wait, **kwargs)
 
 
@@ -474,21 +487,3 @@ def showdir(c, *, to_html=None, printf=True, width=200):
 # 注册进builtins，可以在任意地方直接使用
 setattr(builtins, 'browser', browser)
 setattr(builtins, 'showdir', showdir)
-
-
-def render_echart(ob, name, show=False):
-    """ 渲染显示echart图表
-
-    https://www.yuque.com/xlpr/pyxllib/render_echart
-
-    :param ob: 一个echart图表对象
-    :param name: 存储的文件名
-    :param show: 是否要立即在浏览器显示
-    :return: 存储的文件路径
-    """
-    # 如果没有设置页面标题，则默认采用文件名作为标题
-    if not ob.page_title or ob.page_title == 'Awesome-pyecharts':
-        ob.page_title = name
-    f = ob.render(str(File(f'{name}.html', Dir.TEMP)))
-    if show: browser(f)
-    return f
