@@ -24,10 +24,9 @@ import requests
 
 from pyxllib.prog.newbie import round_int
 from pyxllib.prog.pupil import check_install_package, is_url
-from pyxllib.prog.specialist import XlOsEnv
+from pyxllib.prog.xlosenv import XlOsEnv
 from pyxllib.algo.geo import xywh2ltrb, rect_bounds
 from pyxllib.file.specialist import XlPath, get_etag
-from pyxllib.prog.specialist import TicToc
 from pyxllib.cv.expert import xlcv
 
 
@@ -277,7 +276,7 @@ class XlAiClient:
                     continue
 
             if not connent:
-                raise ConnectionError('PRIU接口登录失败')
+                raise requests.exceptions.ConnectionError('PRIU接口登录失败')
             return connent
         else:
             self._priu_host = hosts[0]  # 不检查的场景，一般都是局域网内使用
@@ -328,6 +327,9 @@ class XlAiClient:
 
     def _divide_options_with_meta(self, options, meta_opts):
         """ 会改变原始输入的两个字典值，但是新返回的才是真正有效的字典 """
+        options = options or {}  # 这里也要先处理，防止options是None类型
+        meta_opts = meta_opts or {}  # 这里也要先处理，防止是None类型
+
         for name in self.META_OPTS_NAME:
             if name in options:
                 meta_opts[name] = options[name]
@@ -792,10 +794,11 @@ class XlAiClient:
         return result_dict
 
     def mixed_multi_vehicle(self, image, **options):
-        result_dict = self.raw_mixed_multi_vehicle(image, options)
+        result_dict = self.raw_mixed_multi_vehicle(image, **options)
+        pprint.pprint(result_dict)
 
         shapes = []
-        for x in result_dict['words_result']:
+        for x in result_dict.get('words_result', []):
             shape = {'label': {}}
             shape.update(loc2points(x['location']))
             shape['label']['text'] = json.dumps({w['word_name']: w['word'] for w in x['license_info']},
@@ -805,8 +808,10 @@ class XlAiClient:
             shapes.append(shape)
 
         result_dict['shapes'] = shapes
-        del result_dict['words_result']
-        del result_dict['words_result_num']
+        if 'words_result' in result_dict:
+            del result_dict['words_result']
+        if 'words_result_num' in result_dict:
+            del result_dict['words_result_num']
         return result_dict
 
     def vehicle_registration_certificate(self, image, **options):
@@ -1042,7 +1047,7 @@ class XlAiClient:
 
         def func(buffer, options):
             image_uri = f'data:image/jpg;base64,' + base64.b64encode(buffer).decode()
-            for i in range(10):  # 尝试10次，10次都失败就无语了~
+            for i in range(3):  # 尝试3次
                 try:
                     r = requests.post('https://api.mathpix.com/v3/latex',
                                       data=json.dumps({'src': image_uri}),
@@ -1073,7 +1078,7 @@ class XlAiClient:
         assert ratio == 1, f'本地不做缩放，由服务器进行缩放处理'
         return buffer.decode()
 
-    def priu_api(self, mode, image=None, texts=None, options=None, **meta_opts):
+    def priu_api(self, mode, image=None, texts=None, options=None, timeout=30, **meta_opts):
         """ 借助服务器来调用该类中含有的其他函数接口
 
         使用该接口的时候，因为服务器一般会有图片等的备份，所以本接口默认不对图片进行备份
@@ -1083,6 +1088,7 @@ class XlAiClient:
         :param image: image、texts、options都是比较常用的几个参数键值，所以显式地写出来
         :param options: 有时候为了严谨，api相关的参数可能要单独打包
             但其实这里的参数混入meta_opts也没事，底层会有divide拆分函数，能区分开不同的参数类型
+        :param timeout: 超时30秒的限制，有些特殊情况可以设置的更短
         :param meta_opts: 也可以自己额外扩展一些键值，兼容一些特殊的输入范式的api接口
             use_exists_xlapi: 是否复用已有的识别结果，避免重复运算，节约api成本
             record_files: 是否保存图片等数据
@@ -1101,12 +1107,17 @@ class XlAiClient:
         if meta_opts:
             data.update(meta_opts)
 
-        # TODO 增加超时timeout=seconds参数？
-        r = requests.post(f'{self._priu_host}/api/{mode}', json=data, headers=self._priu_header)
+        try:
+            r = requests.post(f'{self._priu_host}/api/{mode}',
+                              json=data, headers=self._priu_header, timeout=timeout)
+        except requests.exceptions.Timeout:
+            # 为了方便处理，统一也返回ConnectionError
+            raise requests.exceptions.ConnectionError()
+
         if r.status_code == 200:
             res = json.loads(r.text)
         else:  # TODO 正常状态码不只200，可能还有重定向等某些不一定是错误的状态
-            raise ConnectionError(r.text)
+            raise requests.exceptions.ConnectionError(r.text)
 
         # 2 统一返回值的范式，默认都是dict。 有些特殊格式表示是图片，这里会自动做后处理解析。
         if isinstance(res, dict) and len(res) == 1 and 'imageData' in res:
@@ -1145,7 +1156,7 @@ class XlAiClient:
 
     def hesuan_layout(self, image):
         """ 核酸版面分析 """
-        r = requests.post(f'{self._priu_host}/api/hesuan_layout', headers=self._priu_header,
+        r = requests.post(f'{self._priu_host}/api/hesuan/layout', headers=self._priu_header,
                           json={'image': self._priu_read_image(image)})
         return json.loads(r.text)
 
