@@ -111,14 +111,14 @@ class BasicLabelDataset:
         else:  # 否则直接写入
             file.write_auto(data, **kwargs)
 
-    def writes(self, *, max_workers=8, prt=False, **kwargs):
+    def writes(self, *, max_workers=8, print_mode=False, **kwargs):
         """ 重新写入每份标注文件
 
         可能是内存里修改了数据，需要重新覆盖
         也可能是从coco等其他格式初始化，转换而来的内存数据，需要生成对应的新标注文件
         """
         mtqdm(lambda x: self.write(x, **kwargs), self.rp2data.keys(), desc=f'{self.__class__.__name__}写入标注数据',
-              max_workers=max_workers, disable=not prt)
+              max_workers=max_workers, disable=not print_mode)
 
 
 def __1_labelme():
@@ -152,7 +152,24 @@ def reduce_labelme_jsonfile(jsonpath):
 
     if is_labelme_json_data(data) and data['imageData']:
         data['imageData'] = None
-        XlPath(p).write_auto(data, encoding=encoding, if_exists='replace')
+        XlPath(p).write_json(data, encoding=encoding, if_exists='replace')
+
+
+def reduce_labelme_dir(d, print_mode=False):
+    """ 精简一个目录里的所有labelme json文件 """
+
+    def printf(*args, **kwargs):
+        if print_mode:
+            print(*args, **kwargs)
+
+    i = 0
+    for f in XlPath(d).rglob_files('*.json'):
+        data = f.read_json()
+        if data.get('imageData'):
+            data['imageData'] = None
+            f.write_json(data)
+            i += 1
+            printf(i, f)
 
 
 class ToLabelmeJson:
@@ -808,3 +825,42 @@ class LabelmeDataset(BasicLabelDataset):
         :param reset: 目标目录存在则重置
         """
         pass
+
+
+class ItemFormula:
+    """ 为m2302中科院题库准备的labelme数据相关处理算法
+    如果写到labelme中，其他脚本就复用不了了，所以把核心算法功能写到这里
+
+    line_id的功能是动态计算的，参考了与上一个shape的区别，这里暂时不提供静态算法。
+    """
+
+    @classmethod
+    def check_label(cls, shapes):
+        """ 检查数据异常 """
+        # "删除" 是否都为手写，并且text为空
+
+    @classmethod
+    def joint_label(cls, shapes):
+        """ 将shapes的label拼接成一篇文章
+
+        :return: [line1, line2, ...]
+        """
+        last_line_id = 1
+        paper_text = []
+        line_text = []
+        for sp in shapes:
+            label = json.loads(sp['label'])
+            if label['line_id'] != last_line_id:
+                last_line_id = label['line_id']
+                paper_text.append(' '.join(line_text))
+                line_text = []
+
+            if label['content_class'] == '公式':
+                t = '$' + label['text'] + '$'
+            else:
+                t = label['text']
+            line_text.append(t)
+
+        paper_text.append(' '.join(line_text))
+
+        return paper_text
